#!/usr/bin/env python

from __future__ import print_function
import os
import sys
import re

# USAGE EXAMPLE:
#  xscale_par; grep 'Nano' -A 25 XSCALE.LP; xdscc12 scaled_nonmerged.HKL -dmin 5.0 -dmax 2.5 -nbin 10 > XDSCC.LP; python xdscc.py XDSCC.LP

fin = open("XDSCC.LP")
fin = open(sys.argv[1])
fin_re = open(sys.argv[1])
try:
    cutoff = float(sys.argv[2])
except IndexError:
    cutoff = 1.0


def fill(string, N=15):
    "Pads string with spaces up to length N"
    if len(string) > N:
        return string[:N]
    else:
        return string + " " * (N - len(string))


def get_rejected_crystals(xdscclp, rejection_func=None, mode="noano"):
    """
        Parsing of XDSCC12.LP file using rejection critecia 'rejection_func'.
        Returns set of numbers -- the bad crystals with respect to numbering in
        the initial xdscclp file.
        """

    fin = open(xdscclp)
    bad_crystals = set()
    if mode == "noano":
        a = re.compile("^a\s+")
        b = re.compile("^b\s+")
        c = re.compile("^c\s+")
    elif mode == "ano":
        a = re.compile("^d\s+")
        b = re.compile("^e\s+")
        c = re.compile("^f\s+")
    else:
        print("Wrong mode given to get_rejected_crystals: %s" % mode)
        sys.exit(1)

    while True:
        fline = fin.readline()
        if not fline:
            break

        if a.match(fline):
            crystal_number = int(fline.split()[1])
            if crystal_number % 100 == 0:
                print("Working with crystal number %d" % crystal_number, end="\r")

        elif b.match(fline):
            try:
                CC = [float(i) for i in fline.split()[1:]]
            except ValueError:
                # print("Unusual pattern while parsing CC in %s"%fline)
                CC = resolving(fline.replace("-100", " -100"))
                CC = [i for i in CC if i != 0.0]

        elif c.match(fline):
            Nref = [int(i) for i in fline.split()[1:]]

            CCaverage = sum([i for i in CC]) / len(CC)
            if CCaverage < 0 and sum(Nref) / len(Nref) > 10:
                bad_crystals.add(crystal_number)

    # returns set()
    return bad_crystals


# expressins to parse HKL file
dataset = lambda string: " ISET=" in string and "INPUT_FILE" in string
reflection_file = lambda string: "reflection file is" in string

getnamesfrom = [i for i in fin_re.readlines() if reflection_file(i)][0].split()[-1]
getnamesfrom = open(getnamesfrom)
fin_re.close()

datasets_from_xscale = dict()
i = 1

for fline in getnamesfrom.readlines():
    if dataset(fline):
        # print(fline,end='')
        datasets_from_xscale[i] = {"name": fline.split("INPUT_FILE=")[-1][:-1]}
        i += 1

getnamesfrom.close()


# expressions to parse XDSCC.LP

resolution_shells = lambda string: "resolution shells (for lines starting" in string
abcdef = re.compile("^[abcdef]\s+", re.M)

next_shells = False
for fline in fin.readlines():
    if resolution_shells(fline):
        next_shells = True
        continue
    elif next_shells:
        shells = [float(i) for i in fline.split() if i]
        next_shells = False

    if "overall" in fline:
        j = 0
    elif abcdef.match(fline):
        current_type, numbers = fline.split()[0], [float(i) for i in fline.split()[1:]]
        if current_type == "a" or current_type == "d":
            j += 1
        datasets_from_xscale[j][current_type] = numbers


iterxds = True
try:
    fin = open("iterxds.log")
except IOError:
    iterxds = False

if iterxds:
    i = 1
    for fline in fin.readlines():
        if "overall" in fline and len(fline.split()) > 3:
            rmeas_overall_low = fline.replace("%", "").split()[0]
        elif "XDS_ASCII" in fline:
            name = fline.split()[-1]
            rmeas_low, rmeas_overall = fline.replace("%", "").split()[:2]
            rmeas_low = float(rmeas_low)
            rmeas_overall = float(rmeas_overall)
            for key in datasets_from_xscale.keys():
                if datasets_from_xscale[key]["name"] == name:
                    datasets_from_xscale[key]["rmeas_low"] = rmeas_low
                    datasets_from_xscale[key]["rmeas_overall"] = rmeas_overall


fout = open("good.xdscc", "w")

padding_length = max([len(elem["name"]) for elem in datasets_from_xscale.values()])
toprint = ["--\t%s\t %8.2f" % ("-" * padding_length, 0)]
for key in datasets_from_xscale.keys():
    name = datasets_from_xscale[key]["name"]
    CCnoano = datasets_from_xscale[key]["b"]
    Nrefsnoano = datasets_from_xscale[key]["c"]
    # CCano = datasets_from_xscale[key]['e']
    # Nrefsano = datasets_from_xscale[key]['f']
    if iterxds:
        rmeas_low = datasets_from_xscale[key]["rmeas_low"]
        rmeas_overall = datasets_from_xscale[key]["rmeas_overall"]
        toprint.append(
            "%d\t%s\t %8.2f\t%2.2f\t%2.2f"
            % (
                key,
                fill(name, N=padding_length),
                sum(CCnoano) / len(CCnoano),
                rmeas_low,
                rmeas_overall,
            )
        )
    else:
        toprint.append(
            "%d\t%s\t %8.2f"
            % (key, fill(name, N=padding_length), sum(CCnoano) / len(CCnoano))
        )
        if sum(CCnoano) / len(CCnoano) >= cutoff:
            print("%s" % name, file=fout)


print(*sorted(toprint, key=lambda f: float(f.split()[2])), sep="\n")
